#! /usr/bin/python -O
# -*- coding: iso-8859-15 -*-
#
# Earth17= E17 + XPlanet.
# Copyright (C) 2008 MiKael NAVARRO
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

"""Earth17.
Copyright (C) 2008 MiKael NAVARRO

Generate e17 backgrounds using XPlanet.

"""

# Specific variables for pydoc
__author__ = "MiKael Navarro <klnavarro@gmail.com>"
__date__ = "Sun Nov 16 2008"
__version__ = "0.1.2"
__credits__ = """Thanks to Michal Pasternak for download_clouds.py
and Steve Jones for its initial e17XPlanet project."""

# Include directives
import sys
import os, shutil
import stat
import random
import time
import urllib

# Global variables
XPLANET_PROG = "xplanet"
EDJE_PROG = "edje_cc"
ENLIGHTENMENT_PROG = "enlightenment_remote"
ESETROOT_PROG = "Esetroot"

EARTH17_CONFDIR = os.path.join(os.path.expanduser("~"), ".earth17")

ENLIGHTENMENT_CONFDIR = os.path.join(os.path.expanduser("~"), ".e", "e")
ENLIGHTENMENT_BACKGROUNDS = os.path.join(ENLIGHTENMENT_CONFDIR, "backgrounds")
ENLIGHTENMENT_APPLICATIONS = os.path.join(ENLIGHTENMENT_CONFDIR, "applications", "all")

NICE = 5  # 5 is a nice background level
SLEEP_TIME = 600  # 300s = 5min

# Clouds
CLOUDS_MAP = "earth_clouds.jpg"
# How often to download the image?
DOWNLOAD_FREQUENCY_HOURS = 3
# How many times to retry if download fails (each time tries using a different mirror)
MAX_DOWNLOAD_RETRIES = 3
# The list of mirrors.  Add new ones here.
CLOUDS_MIRRORS = [ "http://www.ruwenzori.net/earth/clouds_2048.jpg",
                   "http://xplanet.dyndns.org/clouds/clouds_2048.jpg",
                   "http://userpage.fu-berlin.de/~jml/clouds_2048.jpg",
                   "http://home.megapass.co.kr/~ysy00/cloud_data/clouds_2048.jpg",
                   "http://home.megapass.co.kr/~holywatr/cloud_data/clouds_2048.jpg",
                   "http://home.megapass.co.kr/~gitto88/cloud_data/clouds_2048.jpg",
                   "http://xplanet.explore-the-world.net/clouds_2048.jpg" ]

# XPlanet
XPLANET_CFG = "xplanet.cfg"
XPLANET_IMG = "earth17.png"

# Enlightenment DR17
EARTH17_EDC = "earth17.edc"
EARTH17_EDJ = "earth17.edj"
EARTH17_DESKTOP = "earth17.desktop"


#
# Download clouds map.
#
def download_clouds(earth17_confdir, clouds_file):
    """Download clouds map for XPlanet.

    Only every DOWNLOAD_FREQUENCY_HOURS"""

    clouds_filename = os.path.join(earth17_confdir, "images", clouds_file)

    if os.access(clouds_filename, os.R_OK and os.W_OK):
        clouds_filename_stat = os.stat(clouds_filename)  # get file size and modification date
        clouds_filename_size = clouds_filename_stat[stat.ST_SIZE]
        clouds_filename_mtime = clouds_filename_stat[stat.ST_MTIME]
        if __debug__: print clouds_filename, clouds_filename_size, clouds_filename_mtime
        
        if (time.time() - clouds_filename_mtime < DOWNLOAD_FREQUENCY_HOURS * 3600
            and clouds_filename_size > 300000):
            return  # already up to date!
        
    for retry in range(MAX_DOWNLOAD_RETRIES):
        clouds_url = CLOUDS_MIRRORS[random.randint(0, len(CLOUDS_MIRRORS)-1)]
        if retry == 0: print "Downloading clouds map '%s' ..." % clouds_url
        else: print "Retrying clouds map '%s' ..." % clouds_url
        try:
            urllib.urlretrieve(clouds_url, clouds_filename)
            break
        except:
            pass

    if os.access(clouds_filename, os.R_OK and os.W_OK):
        print "->", clouds_filename
    else:
        print "No clouds map!"


#
# Generate background image using XPlanet.
#
def generate_xplanet(earth17_confdir, xplanet_cfg, xplanet_img):
    """Generate background image using XPlanet."""

    # Get current screen resolution
    xdpyinfo = os.popen("xdpyinfo | grep dimensions | awk '{print $2}'")
    screen_res = xdpyinfo.readline().strip()
    xdpyinfo.close()

    # Call XPlanet to generate Earth image
    xplanet_cmd = XPLANET_PROG
    xplanet_cmd += " -label_string \"Gaia\" -labelpos +10+10 -body earth -north orbit -radius 60"
    xplanet_cmd += " -longitude 6 -latitude 43.5"
    #xplanet_cmd += " -origin sun -date 20021204.060000"
    xplanet_cmd += " -starmap BSC -range 300"
    xplanet_cmd += " -searchdir %s" % earth17_confdir
    xplanet_cmd += " -geometry %s " % screen_res
    xplanet_cmd += " -output %s -num_times 1" % os.path.join(earth17_confdir, "images", xplanet_img)
    xplanet_cmd += " -config %s" % os.path.join(earth17_confdir, "cfg", xplanet_cfg)
    if __debug__:
        xplanet_cmd += " -verbosity 1"
        print "Exec:", xplanet_cmd
    os.system(xplanet_cmd)


#
# Compile Edje file.
#
def compile_edje(earth17_confdir, xplanet_img):
    """Update earth17.edc and compile earth17.edj file."""

    # Compile Edje file...
    earth17_edc_content = open(os.path.join(earth17_confdir, "cfg", EARTH17_EDC), 'r')
    earth17_edc = open(os.path.join(earth17_confdir, EARTH17_EDC), 'w')
    try:
        print >> earth17_edc, earth17_edc_content.read() % (xplanet_img, xplanet_img)
    finally:
        earth17_edc_content.close()
        earth17_edc.close()

    # Compile Edje and install into E17
    edje_cmd = EDJE_PROG + " -id %s -fd %s %s %s" % (os.path.join(earth17_confdir, "images"),
                                                     os.path.join(earth17_confdir, "fonts"),
                                                     os.path.join(earth17_confdir, EARTH17_EDC),
                                                     os.path.join(ENLIGHTENMENT_BACKGROUNDS, EARTH17_EDJ))
    if __debug__:
        edje_cmd += " -v"
        print "Exec:", edje_cmd
    os.system(edje_cmd)

    
#
# Update backgrounds.
#
def set_backgrounds(earth17_confdir, xplanet_img):
    """Update Enlightenment backgrounds."""

    # Gen desktop file
    earth17_desktop_content = open(os.path.join(earth17_confdir, "cfg", EARTH17_DESKTOP), 'r')
    earth17_desktop = open(os.path.join(earth17_confdir, EARTH17_DESKTOP), 'w')
    try:
        print >> earth17_desktop, earth17_desktop_content.read() % (os.path.join(earth17_confdir, "earth17.py"), os.path.join(earth17_confdir, "cfg", "earth17.png"))
    finally:
        earth17_desktop_content.close()
        earth17_desktop.close()
    # Copy desktop file    
    shutil.copy(os.path.join(earth17_confdir, EARTH17_DESKTOP),
                os.path.join(ENLIGHTENMENT_APPLICATIONS, EARTH17_DESKTOP))
    shutil.copy(os.path.join(earth17_confdir, EARTH17_DESKTOP),
                os.path.join(os.path.expanduser("~"), ".local", "share", "applications", EARTH17_DESKTOP))

    # Default background
    #enlightenment_cmd = ENLIGHTENMENT_PROG + " -default-bg-set %s" % os.path.join(ENLIGHTENMENT_BACKGROUNDS, EARTH17_EDJ)
    #if __debug__:
    #    print "Exec:", enlightenment_cmd

    # Eterm background transparency
    setroot_cmd = ESETROOT_PROG + " " + os.path.join(earth17_confdir, "images", xplanet_img)
    if __debug__:
        print "Exec:", setroot_cmd
    os.system(setroot_cmd)


#
# Main entry point.
#
def main(options):
    """Generate e17 backgrounds using XPlanet:
    - Download coulds image from mirrors;
    - Generate background image using XPlanet;
    - Update earth17.edc and compile edj file;
    - And, update enlightenment backgrounds."""
    
    os.nice(NICE)  # go to background

    while(True):
        # Set the new background, unless enlightenment has been stopped?
        enlightenment_processes = os.popen('pidof enlightenment')
        enlightenment_pid = enlightenment_processes.readline().strip()
        enlightenment_processes.close()
        if not enlightenment_pid:
            print "Enlightenment not running!"
            break
        
        # Set enlightenment backgrounds
        set_backgrounds(options.confdir, XPLANET_IMG)

        # Download clouds map
        download_clouds(options.confdir, CLOUDS_MAP)

        # Generate background image using XPlanet
        generate_xplanet(options.confdir, XPLANET_CFG, XPLANET_IMG)
    
        # Update earth17.edc and compile edj file
        compile_edje(options.confdir, XPLANET_IMG)

        # Wait next run...
        print "Sleep... "
        time.sleep(SLEEP_TIME)

    print "Bye."


#
# External entry point.
#
if __name__ == "__main__":
    # Get options
    from optparse import OptionParser, make_option

    option_list = [
        make_option("", "--install_dir",
                    action="store", dest="confdir", 
                    type="string", default=EARTH17_CONFDIR,
                    help="alternate installation dir (%s)" % EARTH17_CONFDIR),
        make_option("", "--xplanet",
                    action="store", dest="xplanet_cfg", 
                    type="string", default="",
                    help="XPlanet config (%s)" % os.path.join(EARTH17_CONFDIR, XPLANET_CFG)),
        ]

    parser = OptionParser(usage="python -O %prog [options] ...", 
                          version=__version__,
                          description="Generate e17 backgrounds using XPlanet.",
                          option_list=option_list)

    (options, args) = parser.parse_args()

    # Compulsory options
    if not options.confdir:
        paser.error("missing install_dir option!")

    # Process start here
    main(options)
